<?php
/*
Plugin Name: Configurable Tag Cloud
Plugin URI: http://reciprocity.be/ctc/
Description: A tag cloud plugin for Wordpress 2.3 to give you more flexibility with the styling of your tag cloud.
Author: Keith Solomon
Version: 4.1
Author URI: http://reciprocity.be/

	Copyright (c) 2008 Keith Solomon (http://reciprocity.be)
	Configurable Tag Cloud is released under the GNU General Public License (GPL)
	http://www.gnu.org/licenses/gpl.txt
*/

// Generates colors for tag links based on weight (number of posts for tag/category)
// If nothing gets passed in to the function for $mincolor and $maxcolor, it skips the function and uses the defined link color
// from the CSS.  If only one color is passed, it's used for both.
function ColorWeight($weight, $mincolor, $maxcolor) {
	if ($mincolor == "" && $maxcolor == "")
		return;

	if ($mincolor == "") {
		$color = $maxcolor;
		return $color;
	}

	if ($maxcolor == "") {
		$color = $mincolor;
		return $color;
	}

	if ($weight) {
		$weight = $weight/100;

		// Ugly hack to handle CSS shorthand color definitions (i.e., #fff instead of #ffffff)
		if (strlen($mincolor) == 4) {
			$r = substr($mincolor, 1, 1);
			$g = substr($mincolor, 2, 1);
			$b = substr($mincolor, 3, 1);

			$mincolor = "#$r$r$g$g$b$b";
		}

		if (strlen($maxcolor) == 4) {
			$r = substr($maxcolor, 1, 1);
			$g = substr($maxcolor, 2, 1);
			$b = substr($maxcolor, 3, 1);

			$maxcolor = "#$r$r$g$g$b$b";
		}

		$minr = hexdec(substr($mincolor, 1, 2));
		$ming = hexdec(substr($mincolor, 3, 2));
		$minb = hexdec(substr($mincolor, 5, 2));

		$maxr = hexdec(substr($maxcolor, 1, 2));
		$maxg = hexdec(substr($maxcolor, 3, 2));
		$maxb = hexdec(substr($maxcolor, 5, 2));

		$r = dechex(intval((($maxr - $minr) * $weight) + $minr));
		$g = dechex(intval((($maxg - $ming) * $weight) + $ming));
		$b = dechex(intval((($maxb - $minb) * $weight) + $minb));

		if (strlen($r) == 1) $r = "0" . $r;
		if (strlen($g) == 1) $g = "0" . $g;
		if (strlen($b) == 1) $b = "0" . $b;

		$color = "#$r$g$b";
		$color = substr($color,0,7);
		
		return $color;
	}
}

// My tag cloud function - modified to allow changing the colors of the tags based on weight
function ctc( $args = '' ) {
	$defaults = array(
		'smallest' => 8, 'largest' => 22, 'unit' => 'pt', 'number' => 45,
		'format' => 'flat', 'orderby' => 'name', 'order' => 'ASC',
		'exclude' => '', 'include' => '', 'mincolor' => '', 'maxcolor' => '',
		'showcount' => 'no', 'showcats' => 'no', 'empty' => 'no'
	);
	$args = wp_parse_args( $args, $defaults );

	extract($args);

	$tags = get_tags( array_merge($args, array('orderby' => 'count', 'order' => 'DESC')) ); // Always query top tags

	// Now there's categories in the cloud too...
	if ( 'yes' == $showcats ) {
		if ('yes' == $empty) {
			$empty=0;
		} else {
			$empty=1;
		}

		$hide_empty = '&hide_empty=' . $empty;
		
		$cats = get_categories("show_count=1&use_desc_for_title=0&hierarchical=0$hide_empty");

		$tagscats = array_merge($tags, $cats);
	} else {
		$tagscats = array_merge($tags);
	}
	
	if ( empty($tagscats) )
		return;

	$return = generate_tag_cloud( $tagscats, $args ); // Here's where those top tags get sorted according to $args
	if ( is_wp_error( $return ) )
		return false;
	else if (is_array($return)) {
		return $return;
	} else {
		echo apply_filters( 'ctc', $return, $args );
	}
}

// $tagscats = prefetched tag array ( get_tags() & get_categories() )
// $args['format'] = 'flat' => whitespace separated, 'list' => UL, 'array' => array()
// $args['orderby'] = 'name', 'count', 'rand'
function generate_tag_cloud( $tagscats, $args = '' ) {
	global $wp_rewrite;
	$defaults = array(
		'smallest' => 8, 'largest' => 22, 'unit' => 'pt', 'number' => 45,
		'format' => 'flat', 'orderby' => 'name', 'order' => 'ASC',
		'exclude' => '', 'include' => '', 'mincolor' => '', 'maxcolor' => '',
		'showcount' => 'no', 'showcats' => 'no', 'empty' => 'no'
	);
	$args = wp_parse_args( $args, $defaults );

	extract($args);

	if ( !$tagscats )
		return;
	$counts = $tag_links = array();
	if ( 'rand' == $orderby )
		shuffle($tagscats);
	foreach ( (array) $tagscats as $tag ) {
		$counts[$tag->name] = $tag->count;
		$cat = $tag->taxonomy;
		if ('category' == $cat) {
			$tag_links[$tag->name] = get_category_link( $tag->term_id );
		} else {
			$tag_links[$tag->name] = get_tag_link( $tag->term_id );
		}
		if ( is_wp_error( $tag_links[$tag->name] ) )
			return $tag_links[$tag->name];
		$tag_ids[$tag->name] = $tag->term_id;
	}

	$min_count = min($counts);
	$spread = max($counts) - $min_count;
	if ( $spread <= 0 )
		$spread = 1;
	$font_spread = $largest - $smallest;
	if ( $font_spread <= 0 )
		$font_spread = 1;
	$font_step = $font_spread / $spread;

	// SQL cannot save you; this is a second (potentially different) sort on a subset of data.
	if ( 'name' == $orderby )
		uksort($counts, 'strnatcasecmp');
	elseif ( 'count' == $orderby )
		asort($counts);

	if ( 'DESC' == $order )
		$counts = array_reverse( $counts, true );

	$a = array();

	$rel = ( is_object($wp_rewrite) && $wp_rewrite->using_permalinks() ) ? ' rel="tag"' : '';

	foreach ( $counts as $tag => $count ) {
		if ( $largest == $smallest )
			$tag_weight = $largest;
		else
			$tag_weight = ($smallest+(($count-$min_count)*$font_step));
		$diff = $largest-$smallest;
		if ( $diff <= 0 )
			$diff = 1;
		if ( 'yes' == $showcount )
			$postcount = '('.$count.')';
		$color_weight = round(99*($tag_weight-$smallest)/($diff)+1);
		$tag_color = ColorWeight($color_weight, $mincolor, $maxcolor);
		$tag_id = $tag_ids[$tag];
		$tag_link = clean_url($tag_links[$tag]);
		$tag = str_replace(' ', '&nbsp;', wp_specialchars( $tag ));
		$a[] = "<a href=\"$tag_link\" class=\"tag-link-$tag_id\" title=\"" . attribute_escape( sprintf( __('%d topics'), $count ) ) . "\"$rel style=\"font-size: " . $tag_weight
			. "$unit;".(isset($tag_color) ? " color: $tag_color;" : "")
			. "\">$tag".('yes' == $showcount ? " $postcount" : "")."</a>";
	}

	switch ( $format ) :
	case 'array' :
		$return =& $a;
		break;
	case 'list' :
		$return = "<ul class='wp-tag-cloud'>\n\t<li>";
		$return .= join("</li>\n\t<li>", $a);
		$return .= "</li>\n</ul>\n";
		break;
	default :
		$return = join("\n", $a);
		break;
	endswitch;

	return apply_filters( 'generate_tag_cloud', $return, $tagscats, $args );
}

// This gets called at the plugins_loaded action
function widget_ctc_init() {
	
	// Check for the required API functions
	if ( !function_exists('register_sidebar_widget') || !function_exists('register_widget_control') )
		return;

	// This saves options and prints the widget's config form.
	function widget_ctc_control() {
		$options = $newoptions = get_option('widget_ctc');
		if ( $_POST['ctc-submit'] ) {
			$newoptions['title'] = strip_tags(stripslashes($_POST['ctc-title']));
			$newoptions['number'] = strip_tags(stripslashes($_POST['ctc-number']));
			$newoptions['unit'] = strip_tags(stripslashes($_POST['ctc-unit']));
			$newoptions['smallest'] = strip_tags(stripslashes($_POST['ctc-smallest']));
			$newoptions['largest'] = strip_tags(stripslashes($_POST['ctc-largest']));
			$newoptions['mincolor'] = strip_tags(stripslashes($_POST['ctc-mincolor']));
			$newoptions['maxcolor'] = strip_tags(stripslashes($_POST['ctc-maxcolor']));
			$newoptions['format'] = strip_tags(stripslashes($_POST['ctc-format']));
			$newoptions['orderby'] = strip_tags(stripslashes($_POST['ctc-orderby']));
			$newoptions['order'] = strip_tags(stripslashes($_POST['ctc-order']));
			$newoptions['showcount'] = strip_tags(stripslashes($_POST['ctc-showcount']));
			$newoptions['showcats'] = strip_tags(stripslashes($_POST['ctc-showcats']));
			$newoptions['empty'] = strip_tags(stripslashes($_POST['ctc-empty']));
			//$newoptions['tags'] = explode(' ', trim(strip_tags(stripslashes($_POST['ctc-tags']))));
		}
		if ( $options != $newoptions ) {
			$options = $newoptions;
			update_option('widget_ctc', $options);
		}
		$title = htmlspecialchars($options['title'], ENT_QUOTES);
		$number = htmlspecialchars($options['number'], ENT_QUOTES);
		$unit = htmlspecialchars($options['unit'], ENT_QUOTES);
		$smallest = htmlspecialchars($options['smallest'], ENT_QUOTES);
		$largest = htmlspecialchars($options['largest'], ENT_QUOTES);
		$mincolor = htmlspecialchars($options['mincolor'], ENT_QUOTES);
		$maxcolor = htmlspecialchars($options['maxcolor'], ENT_QUOTES);
		$format = htmlspecialchars($options['format'], ENT_QUOTES);
		$orderby = htmlspecialchars($options['orderby'], ENT_QUOTES);
		$order = htmlspecialchars($options['order'], ENT_QUOTES);
		$showcount = htmlspecialchars($options['showcount'], ENT_QUOTES);
		$showcats = htmlspecialchars($options['showcats'], ENT_QUOTES);
		$empty = htmlspecialchars($options['empty'], ENT_QUOTES);
?>
	
	<div style="text-align:center">
		<h3>Configurable Tag Cloud Options</h3>
		<span style="line-height:15px"><br /><br /></span>
		<table>
			<tr><td><strong>Title</strong></td>
				<td><input style="text-align:right" type="text" id="ctc-title" name="ctc-title" value="<?php echo wp_specialchars($options['title'], true); ?>" /></td>
				<td style="font-size:0.75em">Title shown in sidebar.</td>
			</tr>
			<tr><td><strong>Number of Tags to Display</strong></td>
				<td><input style="text-align:right" type="text" id="ctc-number" name="ctc-number" value="<?php echo wp_specialchars($options['number'], true); ?>" /></td>
				<td style="font-size:0.75em">Controls the total number of tags in your cloud.</td>
			</tr>
			<tr><td><strong>Font Display Unit</strong></td>
				<td style="text-align:right;">
					<select id="ctc-unit" name="ctc-unit" size="1" value="" />
			   			<option value="px" <?=($unit=="px")?'selected':''?>>Pixel</option>
			   			<option value="pt" <?=($unit=="pt")?'selected':''?>>Point</option>
			   			<option value="em" <?=($unit=="em")?'selected':''?>>Em</option>
				   		<option value="%" <?=($unit=="%")?'selected':''?>>Percent</option>
		   			</select>
				</td>
				<td style="font-size:0.75em">What unit to use for font sizes.</td>
			</tr>
			<tr><td><strong>Smallest Font Size</strong></td>
				<td><input style="text-align:right" type="text" id="ctc-smallest" name="ctc-smallest" value="<?php echo wp_specialchars($options['smallest'], true); ?>" /></td>
				<td style="font-size:0.75em">Tags will be displayed no smaller than this value.</td>
			</tr>
			<tr><td><strong>Largest Font Size</strong></td>
				<td><input style="text-align:right" type="text" id="ctc-largest" name="ctc-largest" value="<?php echo wp_specialchars($options['largest'], true); ?>" /></td>
				<td style="font-size:0.75em">Tags will be displayed no larger that this value.</td>
			</tr>
			<tr><td><strong>Min. Tag Color</strong></td>
				<td><input style="text-align:right" type="text" id="ctc-mincolor" name="ctc-mincolor" value="<?php echo wp_specialchars($options['mincolor'], true); ?>" /></td>
				<td style="font-size:0.75em">Beginning color for tag gradient.  Please include the #.</td>
			</tr>
			<tr><td><strong>Max. Tag Color</strong></td>
				<td><input style="text-align:right" type="text" id="ctc-maxcolor" name="ctc-maxcolor" value="<?php echo wp_specialchars($options['maxcolor'], true); ?>" /></td>
				<td style="font-size:0.75em">Ending color for tag gradient.  Please include the #.</td>
			</tr>
			<tr><td><strong>Cloud Format</strong></td>
				<td style="text-align:right;">
					<select id="ctc-format" name="ctc-format" size="1" value="" />
				   		<option value="flat" <?=($format=="flat")?'selected':''?>>Flat</option>
				   		<option value="list" <?=($format=="list")?'selected':''?>>List</option>
				   		<option value="array" <?=($format=="array")?'selected':''?>>Array</option>
		   			</select>
				</td>
				<td style="font-size:0.75em">How to display the cloud.</td>
			</tr>
			<tr><td><strong>Categories in cloud?</strong></td>
				<td><input type="radio" id="ctc-showcats" name="ctc-showcats" <?=($showcats=="yes")?'checked="checked"':''?> value="yes" /> Yes <input type="radio" id="ctc-showcats" name="ctc-showcats" <?=($showcats=="no")?'checked="checked"':''?> value="no" /> No</td>
				<td style="font-size:0.75em">Display categories in cloud.</td>
			</tr>
			<tr><td><strong>Show Empty?</strong></td>
				<td><input type="radio" id="ctc-empty" name="ctc-empty" <?=($empty=="yes")?'checked="checked"':''?> value="yes" /> Yes <input type="radio" id="ctc-empty" name="ctc-empty" <?=($empty=="no")?'checked="checked"':''?> value="no" /> No</td>
				<td style="font-size:0.75em">Display empty categories in cloud.</td>
			</tr>
			<tr><td><strong>Display Post Count?</strong></td>
				<td><input type="radio" id="ctc-showcount" name="ctc-showcount" <?=($showcount=="yes")?'checked="checked"':''?> value="yes" /> Yes <input type="radio" id="ctc-showcount" name="ctc-showcount" <?=($showcount=="no")?'checked="checked"':''?> value="no" /> No</td>
				<td style="font-size:0.75em">Show number of posts for each tag.</td>
			</tr>
			<tr><td><strong>Sort By</strong></td>
				<td style="text-align:right;">
					<select id="ctc-orderby" name="ctc-orderby" size="1" value="" />
				   		<option value="name" <?=($orderby=="name")?'selected':''?>>Name</option>
				   		<option value="count" <?=($orderby=="count")?'selected':''?>>Count</option>
				   		<option value="rand" <?=($orderby=="rand")?'selected':''?>>Random</option>
		   			</select>
				</td>
				<td style="font-size:0.75em">What field to sort by.</td>
			</tr>
			<tr><td><strong>Sort Order</strong></td>
				<td style="font-size:0.85em"><input type="radio" id="ctc-order" name="ctc-order" <?=($order=="ASC")?'checked="checked"':''?> value="ASC" /> Ascending <input type="radio" id="ctc-order" name="ctc-order" <?=($order=="DESC")?'checked="checked"':''?> value="DESC" /> Descending</td>
				<td style="font-size:0.75em">Direction of sort.</td>
			</tr>
			<!-- <tr><td><strong>Tags To Exclude</strong></td>
				<td>
					<input style="text-align:right" type="text" id="ctc-tags" name="ctc-tags" value="<?php echo wp_specialchars(implode(' ', (array) $options['tags']), true); ?>" />
				</td>
				<td style="font-size:0.75em">Optional</td>
			</tr> -->
		</table>
		<input type="hidden" name="ctc-submit" id="ctc-submit" value="1" />
	</div>

<?php
	}

	// The widget itself
	function widget_ctc($args) {
	extract($args);
	$defaults = array('title' => 'Tags', 'number' => '45', 'unit' => 'pt', 'smallest' => 8, 'largest' => 22, 'format' => 'flat', 'orderby' => 'name', 'order' => 'ASC', 'mincolor' => '', 'maxcolor' => '', 'showcount' => 'no', 'showcats' => 'no', 'empty' => 'no');
	$options = (array) get_option('widget_ctc');

	foreach ( $defaults as $key => $value )
		if ( $options[$key] == "" )
			$options[$key] = $defaults[$key];

		$tagcloud = 'smallest='.$options['smallest'];
		$tagcloud.= '&largest='.$options['largest'];
		$tagcloud.= '&mincolor='.$options['mincolor'];
		$tagcloud.= '&maxcolor='.$options['maxcolor'];
		$tagcloud.= '&unit='.$options['unit'];
		$tagcloud.= '&format='.$options['format'];
		$tagcloud.= '&number='.$options['number'];
		$tagcloud.= '&orderby='.$options['orderby'];
		$tagcloud.= '&order='.$options['order'];
		$tagcloud.= '&showcount='.$options['showcount'];
		$tagcloud.= '&showcats='.$options['showcats'];
		$tagcloud.= '&empty='.$options['empty'];
		//$tagcloud.= count($options['tags']) ? '&exclude='.implode(',', $options['tags']) : '';
		
		//echo 'tag cloud string: '.$tagcloud.'<br />';

	echo $before_widget;
	echo $before_title . $options['title'] . $after_title;
	echo '<div class="ctc">';
		ctc($tagcloud);
	echo '</div>';
	echo $after_widget;
}

	// Tell Dynamic Sidebar about our new widget and its control
	register_sidebar_widget(array('CTC', 'widgets'), 'widget_ctc');
	register_widget_control(array('CTC', 'widgets'), 'widget_ctc_control', 420, 510);
	
}

// Delay plugin execution to ensure Dynamic Sidebar has a chance to load first
add_action('widgets_init', 'widget_ctc_init');

?>